package fs_test

import (
	"context"
	"testing"
	"testing/fstest"

	"github.com/google/go-cmp/cmp"
	"golang.org/x/text/language"

	"source.toby3d.me/toby3d/home/internal/domain"
	repository "source.toby3d.me/toby3d/home/internal/site/repository/fs"
)

func TestGet(t *testing.T) {
	t.Parallel()

	testData := fstest.MapFS{
		"index.ru.md": &fstest.MapFile{
			Data: []byte("---\ntitle: пример\n---\nпривет, мир!\n"),
		},
		"index.en.md": &fstest.MapFile{
			Data: []byte("---\ntitle: example\n---\nhello, world!\n"),
		},
	}

	repo := repository.NewFileSystemSiteRepository(testData)

	for name, tc := range map[string]struct {
		input  language.Tag
		expect *domain.Site
	}{
		"english": {
			input: language.English,
			expect: &domain.Site{
				Language: language.English,
				Title:    "example",
				Params:   make(map[string]any),
			},
		},
		"russian": {
			input: language.Russian,
			expect: &domain.Site{
				Language: language.Russian,
				Title:    "пример",
				Params:   make(map[string]any),
			},
		},
	} {
		name, tc := name, tc

		t.Run(name, func(t *testing.T) {
			t.Parallel()

			out, err := repo.Get(context.Background(), tc.input)
			if err != nil {
				t.Fatal(err)
			}

			if diff := cmp.Diff(out, tc.expect, cmp.AllowUnexported(language.Und)); diff != "" {
				t.Error(diff)
			}
		})
	}
}
